/* REXX */

SAY
SAY "IP-Nodes v0.20 - 2003-10-23  by  Hiro Dudani, 2:244/1512"
SAY
/*
 * Creates BinkD include file and route file from nodelist.
 *
 * Requires RxSock - REXX function package for tcp/ip sockets
 *
 * Invocation:
 *  ip-nodes <nodelist> [<output nodelist> [<output routefile>]] [-c <cfgfile>]
 *
 * Arguments:
 *
 *  <nodelist>         : [Path and] filename of FTN nodelist
 *                        (May include wildcards)
 *  <output nodelist>  : [Path and] filename for BinkD include file
 *                        Use . or leave empty for default
 *                        Use NUL: for "don't create"
 *  <output routefile> : [Path and] filename for routing file
 *                        Use . or leave empty for default
 *                        Use NUL: for "don't create"
 *  <cfgfile>          : [Path and] filename of configuration file
 *                        Use . to disable default
 *
 * Examples:
 *  ip-nodes F:\Fido\Nodelist\NODELIST.269 ibnnodes.inc routes.inc
 *  ip-nodes NODELIST.*
 *  ip-nodes Nodelist\Region24.269 . NUL: -c E:\Fido\etc\ip-nodes.cfg
 *
 */

/* Verbrochen by:
 * Hiro Dudani, 2:244/1512  (FoIP: The.Geek2-Grotto.de)                */

/***********************************************************************/
/* Change these variables as needed:                                   */
/***********************************************************************/

/* Default configuration file (e.g. "ip-nodes.cfg") */
/* Use "" for no config file. */
CONFIGFILE="ip-nodes.cfg"


/* ---------------------------------------------------------------------- */
/* All following settings can be overridden from the configuration file:  */
/* ---------------------------------------------------------------------- */

/* Default output file (e.g. "ibnnodes.inc" or "BINKD.TXT") */
/* Use "" for no output file.  */
DEFAULTNODESFILE="ibnnodes.inc"

/* Default output route file (e.g. "routes.inc") */
/* Again, use "" for no route file. */
DEFAULTROUTEFILE="routes.inc"


/* Do DNS lookups of host names? (Much faster, but dangerous if you use
 *                                a route file) */
/* Use 0 for no or 1 for yes. */
DoLookups=1

/* Template for route file lines. All occurrences of RoutePlaceholder will
 * be replaced by the 3D address of the node.
 * (e.g. RoutePlaceholder="%%"
 *       RouteTemplate="route to %%.* via %%" ) */
RoutePlaceholder="%%"
RouteTemplate="route direct no-route %%"

/* Options to be added to every node line (e.g. "-crypt") */
NODEOPTS=""

/* Your Zone (only needed if your input file doesn't have a Zone entry,
 *            like a Region or Network list) */
Zone=2

/* Root domain for * (pfnz) (e.g. "fidonet.net" ) */
/* Use "-" to disable. */
DEFAULTDOMAIN="fidonet.net"


/* Nodelist flag for Protocol (e.g. "IBN") */
NLFLAG="IBN"

/* Nodelist flag for Internet address (e.g. "INA") */
INAFLAG="INA"


/* Expected maximum length of 3D FTN addresses (cosmetic only) */
AddrLength=11


/* -------------------------------------------------------------------- *
 *
 * HostName:
 * Here you can "preload" host names for nodes if you know more than
 * the Nodelist does. These will be put before anything found in the
 * input list or used alone if the node isn't listed.
 *
 * Note:
 * This will not be checked for correctness, dupes or anything.
 *
 * Example:
 * HostName.2.299.999="host.example.com:24555"
 *
 */



/*
 * PostName:
 * This is the same as HostName, but these will be put *after* the input
 * list data.
 *
 * Example:
 * PostName.2.299.999="backup.example.com;*"
 *
 */

PostName.2.244.1512="The1.Geek2-Grotto.de;grotto.fidosoft.de;The3.Geek2-Grotto.de"


/***********************************************************************/
/**************** ! No User Servicable Parts Below ! *******************/
/***********************************************************************/

PARSE ARG infile outfile routefile "-c" cfgfile

IF infile = "" THEN DO
  SAY
  SAY "IP-Nodes: Please read the beginning of this file for usage instructions."
  EXIT 1
END

IF cfgfile \= "" THEN CONFIGFILE=strip(cfgfile)
IF cfgfile = "." THEN CONFIGFILE=""

/* Read configuration file */
IF (CONFIGFILE \= "") THEN
 DO
  IF STREAM(CONFIGFILE,'C','OPEN READ') = "READY:" THEN
   DO
    SAY "Reading configuration file " || CONFIGFILE
    DO WHILE lines(CONFIGFILE)
      CfgLine=strip(changestr(D2C(9),linein(CONFIGFILE),' '))
      IF CfgLine = "" THEN ITERATE                           /* Empty line */
      IF VERIFY(LEFT(CfgLine,1),'#;/') = 0 THEN ITERATE      /* Comment line */
      PARSE VAR CfgLine CfgKeyword CfgContent
      INTERPRET strip(CfgKeyword) || '="' || strip(CfgContent) || '"'
    END
    CfgStream=STREAM(CONFIGFILE,'C','CLOSE')
    SAY "Done."
   END
   ELSE DO
    SAY; SAY "Error opening config file " || CONFIGFILE || " for reading."
    SAY "Continuing with built-in configuration..."; SAY;
   END
 END

infile=getFilename(infile)

IF ((outfile = "") | (outfile = ".")) THEN outfile=DEFAULTNODESFILE
IF ((routefile = "") | (routefile =".")) THEN routefile=DEFAULTROUTEFILE

NLFLAG = "," || NLFLAG
INAFLAG = "," || INAFLAG

SAY
SAY "Opening input file    " || infile
IF STREAM(infile,'C','OPEN READ') \= "READY:" THEN DO
 SAY; SAY "Error opening input file " || infile || " for reading."
 SAY "Exiting."
 EXIT 1
END

IF outfile \= "" THEN DO
 SAY "Opening output file   " || outfile
 IF STREAM(outfile,'C','OPEN REPLACE NOBUFFER') \= "READY:" THEN DO
  SAY; SAY "Error opening output file " || outfile || " for writing."
  SAY "Exiting."
  EXIT 1
 END
END

IF routefile \= "" THEN DO
 SAY "Opening routing file  " || routefile
 IF STREAM(routefile,'C','OPEN REPLACE NOBUFFER') \= "READY:" THEN DO
  SAY; SAY "Error opening route file " || routefile || " for writing."
  SAY "Exiting."
  EXIT 1
 END
END

IF DoLookups = 1 THEN DO
  SAY
  CALL RxFuncAdd "SockLoadFuncs","rxSock","SockLoadFuncs"
  CALL SockLoadFuncs
END

IF outfile \= "" THEN Call LineOut OutFile,"#  " || DATE() || "  " || TIME()

Net=0
Node=0

DO WHILE LINES(infile)
  NStatus=""; NNode=""; NName=""; NSysOp=""; NPhone=""; NFlags="";
  PARSE VALUE LINEIN(infile) WITH NStatus "," NNode "," NName "," . "," NSysOp "," NPhone "," . "," NFlags ;

  IF LEFT(NStatus,1) = ";" THEN ITERATE

  IF NStatus = "Zone" THEN DO
                             Zone=NNode;
                             Net=NNode;
                             Node=0;
                             CALL CharOut ,D2C(13) || COPIES(" ",79)
                             SAY
                             SAY "Processing Zone " || Zone || ":*";
                           END
                      ELSE Node=NNode

  IF NStatus = "Region" THEN DO
                             Net=NNode;
                             Node=0;
                             CALL CharOut ,D2C(13) || COPIES(" ",79)
                             SAY
                             SAY "Processing Region " || Zone || ":" || Net || "/*";
                           END

  IF NStatus = "Host" THEN DO
                             Net=NNode;
                             Node=0;
                             CALL CharOut ,D2C(13) || COPIES(" ",79) || D2C(13)
                             SAY "Processing Net " || Zone || ":" || Net || "/*";
                           END

  NFlags = "," || NFlags
  IF (COUNTSTR(NLFLAG,NFlags) = 0) & (VAR('HostName.Zone.Net.Node') = 0) & (VAR('PostName.Zone.Net.Node') = 0) THEN ITERATE
  IF (TRANSLATE(NStatus) = "HOLD") | (TRANSLATE(NStatus) = "DOWN") THEN ITERATE


  Addr3D = Zone || ":" || Net || "/" || Node
  pfnz = "f" || Node || ".n" || Net || ".z" || Zone || "." || defaultdomain


  NHostPhone=""
  NHostName=""
  NHostFlag=""

  NHost=""
  Port=""

  IF VAR('HostName.Zone.Net.Node') THEN NHost = ";" || HostName.Zone.Net.Node

 /* Get IP# from phone field */
  IF LEFT(NPhone,4) = "000-" THEN
    IF isHost(CHANGESTR("-",DELSTR(NPhone,1,4),".") DoLookups) THEN
      NHostPhone=CHANGESTR("-",DELSTR(NPhone,1,4),".")          /* <- */

 /* Get host name from system name field */
  IF isHost(NName DoLookups) THEN NHostName=NName                         /* <- */

 /* Get host name from INA flag */
  FlagHost = flagContent(NFlags INAFLAG)
  IF isHost(FlagHost DoLookups) THEN NHostFlag=FlagHost                   /* <- */

  FlagHost=""

 /* Get host name or port from protocol flag */
  NLContent=flagContent(NFlags NLFLAG)
  IF POS(":",NLContent) = 0 THEN        /* 'Host' or 'Port' */
    DO
      IF DATATYPE(NLContent,9) THEN
        Port = ":" || NLContent                                 /* {- */
       ELSE
        IF isHost(NLContent DoLookups) THEN
          NHostFlag = NLContent                                 /* <- */
    END
   ELSE                                 /* 'Host:Port' */
    DO
      PARSE VALUE NLContent WITH FlagHost ":" FlagPort
      IF DATATYPE(FlagPort,9) THEN
        IF isHost(FlagHost DoLookups) THEN
          DO
            NHostFlag = FlagHost                                /* <- */
            Port = ":" || FlagPort                              /* {- */
          END
    END


 /* Remove duplicates */
  IF (NHostFlag = NHostPhone) | (NHostFlag = NHostName) THEN
    NHostFlag=""
  IF NHostName = NHostPhone THEN
    NHostName=""

 /* Put it all together */
  IF NHostPhone \= "" THEN NHost = NHost || ";" || NHostPhone || Port
  IF NHostName  \= "" THEN NHost = NHost || ";" || NHostName || Port
  IF NHostFlag  \= "" THEN NHost = NHost || ";" || NHostFlag || Port

  IF VAR('PostName.Zone.Net.Node') THEN NHost = NHost || ";" || PostName.Zone.Net.Node

  NHost = DELSTR(NHost,1,1)


 /* If nothing was found, try pfnz.DEFAULTDOMAIN */
  IF NHost="" THEN IF isHost(pfnz DoLookups) THEN NHost="*" || Port


  padspace=COPIES(" ",ABS(AddrLength-LENGTH(Addr3D)))

 /* Write output files */

  IF outfile \= "" THEN DO
    nodeline = "node " || Addr3D || padspace || " " || NODEOPTS || " " || NHost || " -"
    IF NHost = "" THEN nodeline = "#E" Addr3D NName Port NSysOp
    Call LINEOUT outfile,nodeline
  END

  IF routefile \= "" THEN DO
    IF NHost \= "" THEN DO
      routeline  = CHANGESTR(RoutePlaceholder,RouteTemplate,Addr3D)
      Call LINEOUT routefile,routeline
    END
  END

END


IF outfile \= "" THEN Call LineOut OutFile,"#  " || DATE() || "  " || TIME()

SAY; SAY "Done. Closing files."
InStream=STREAM(infile,'C','CLOSE')
IF outfile \= "" THEN OutStream=STREAM(outfile,'C','CLOSE')
IF routefile \= "" THEN RouteStream=STREAM(routefile,'C','CLOSE')

EXIT

/* ======================================================================== */

isIP: PROCEDURE
  ARG arg1

  IF COUNTSTR(".",arg1) <> 3 THEN RETURN 0
  IF VERIFY(arg1,"0123456789.") > 0 THEN RETURN 0

  PARSE VAR arg1 quad1 "." quad2 "." quad3 "." quad4
  IF \DATATYPE(quad1,9) THEN RETURN 0
  IF \DATATYPE(quad2,9) THEN RETURN 0
  IF \DATATYPE(quad3,9) THEN RETURN 0
  IF \DATATYPE(quad4,9) THEN RETURN 0
  IF quad1 < 1 THEN RETURN 0
  IF quad2 < 0 THEN RETURN 0
  IF quad3 < 0 THEN RETURN 0
  IF quad4 < 0 THEN RETURN 0
  IF quad1 > 255 THEN RETURN 0
  IF quad2 > 255 THEN RETURN 0
  IF quad3 > 255 THEN RETURN 0
  IF quad4 > 255 THEN RETURN 0

  RETURN 1

/* ------------------------------------------------------------------------ */

isHost: PROCEDURE
  PARSE ARG HostPart DoLook

  Letters = XRANGE('A','Z') || XRANGE('a','z')
  Digits  = XRANGE('0','9')
  Punct   = "-."
  AllowedChars = Letters || Digits || Punct

  IF HostPart = "" THEN RETURN 0

  IF isIP(HostPart) THEN RETURN 1

  IF VERIFY(HostPart,"0123456789.") = 0 THEN RETURN 0    /* Looks like an IP, but isn't. */
  IF COUNTSTR(".",HostPart) = 0 THEN RETURN 0            /* An FQDN must have at least one "." */
  IF VERIFY(LEFT(HostPart,1),Punct) = 0 THEN RETURN 0    /* Host names must begin with a letter (or digit)... */
  IF RIGHT(HostPart,1) = "-" THEN RETURN 0               /* ...end in a letter, digit or "."... */
  IF VERIFY(HostPart,AllowedChars) > 0 THEN RETURN 0     /* ...and consist only of AllowedChars. */

  CALL CharOut ,D2C(13) || COPIES(" ",79) || D2C(13)
  Call CharOut ,HostPart

  IF DoLook = 1 THEN
    IF SockGetHostByName(HostPart,.) = 0 THEN RETURN 0

  RETURN 1

/* ------------------------------------------------------------------------ */

flagContent: PROCEDURE
  PARSE ARG aNFlags aNLFLAG

  Content = ""

  FlagPos = POS(aNLFLAG || ":",aNFlags)
  IF FlagPos > 0 THEN DO
    EndFlag = POS(",",aNFlags,FlagPos+1) - 1
    IF EndFlag <= FlagPos THEN EndFlag = LENGTH(aNFlags)
    ContentPos = FlagPos + LENGTH(aNLFLAG) + 1
    Content = SUBSTR(aNFlags,ContentPos,EndFlag-ContentPos+1)
  END

  RETURN Content

/* ------------------------------------------------------------------------ */

getFilename: PROCEDURE
 PARSE ARG infile

 outfile=infile

 IF verify(infile,'*?','M') > 0 THEN DO
   Call RxFuncAdd "SysLoadFuncs","RexxUtil","SysLoadFuncs"
   Call SysLoadFuncs

   Call SysFileTree infile,'filestem','FO'
   filenum=filestem.0

   IF filenum > 0 THEN outfile=filestem.filenum

  END

 RETURN outfile

/* ------------------------------------------------------------------------ */

/*

Type Bits/KeyID    Date       User ID
pub  2047/962CC1B5 1996/04/05 Hiro Dudani <dudani@gmx.de>

-----BEGIN PGP PUBLIC KEY BLOCK-----
Version: 2.6.3in

mQENAzFlrVYAAAEH/1jsIRqDU4LIGM7DNQabor0OYVoPJ+FQ+GAnjAvSrJb56e5w
Hzg9F3cicCzDhRucuUlkqyY3/PeZbwabr6G2x3a/1XrVTMtW3vYIEZ5LGxeLdEEh
hSNjFkURwnCDJt9i1svC5qhKJx8KQcmbTTAcABTHcgts42zPDrcrx6Svx/lfwPB/
e5BCAt6NPhU1awznm6LU07Rxu6ZyoufE1Vj9DxFnXJsvSxNKNNc6ngxMpaP+ae7d
tcMUwsrDVF1UeSxMP/tqtnPB7+/lUmuKoSu6rw+lOWKzNMBPITFlpOaTFD94Qxmx
GeqCNVop3/1lxwzNsYD1JTT2jzZQL5KneZYswbUABRG0G0hpcm8gRHVkYW5pIDxk
dWRhbmlAZ214LmRlPokBFQMFEDVR89kugTHYZDSbsQEBghMIAMgYpits5L1gPvrN
dZ4pdEg1rsc0Jng10GMkdZ9gzBLIP9bxlsPeFgUfyAaFaqtzEi0HeHPkX7BEkNX7
TDbM7qhK2HWtD9YsO9xFKGiu0/PjWxKW+13ylr/2igQyAjFAoEocHpDhleXOSBJ2
IgT+++AB1ebdKfWzI/0MhO4jg7Tx+HufwH9inBG+pnymAkUz5k9/zS4ZTdV7F0Qf
o8ID2TpPvReeQhs1NuSdqt6j8a2bM4mjvkwEbFizH5Wf+1TL/ojRNCTIOXLsO0tv
sTMrr9DHrNG6Vpgy6zpAFwovLTs55HOys78mx6uXZrodVlOKUlNIm3GZ4Z2PavSr
oWBDCH+JARUDBRAz/Y5O1lRbHM/BAPUBAXxwCACYdZsI9a/nPn7EdkUeY0yJHLev
eEeFaJPSS8bQwO4oeJ1XORdYjWNmVN2zD+IuJYb2xQZWqzODXkTF1RwWXnmBmVEI
rjogRlWqfbO8TzbeMRsWoGGqZn7E4Et8faqeJEcHP+CcMhbiPqQvgNxbfY9G+MbV
HSAUScXCig/M3jdAHXZOsExNeU3HgmseXlOWu9yiAnSnlDl8ISP8A7c/y8dIQFkw
QgIl2ePSdjn/5FVaO8dWuCbm5BlZBXFML95M4VNVml3TmtkeR/Yor7wUqAADabC8
vywEuhX/upJcbBWhmh36/2OZVGsHWThhvYHnsMUz0xBU5bPN6+UCCJSKRMpciQEV
AwUQM9ithi+Sp3mWLMG1AQEe1gf+Kp4DBUg9jady43ZIcF9XJyGk6a9QWFfH4n4Q
+BlfbRFTgh5jjrQo13PmUUx3jkU0JZd9C3NssAuApvwyk5ktMD5v8CA9aVMvw+Nv
O768Gw9OGuQrgmTJ7l0dhamB1KgZQY/ZWtSKBnCPTNJO/FfChVA4NJRfBf90FKt/
aDRpf1UO0WGAQ60FAtr666/fcmaSecKCw57mRebVwlR253mSLB5FF5vDZZPGkc2l
E5VKU2Y4Uz3JI4q9kJvs92joXvYq3PdgHPgC0Od09fonjsrpH8WMONKXhX/Wdsj1
GDfH++ectK2Vuox8PwpDRLVjy9EfoaaELYeeSOgIfNgtsRJO7g==
=R3cn
-----END PGP PUBLIC KEY BLOCK-----

*/
